#include "base_measure.h"

namespace Localization {
namespace Ego {
// 状态量交叉校验
bool BaseMeasure::isStatic_ = false;
bool BaseMeasure::isImuValid_ = false;
bool BaseMeasure::isVehValid_ = false;
bool BaseMeasure::isRtkValid_ = false;
bool BaseMeasure::isVlocValid_ = false;
bool BaseMeasure::isImuBlock_ = false;
bool BaseMeasure::isVehBlock_ = false;
bool BaseMeasure::isRtkBlock_ = false;
bool BaseMeasure::isVlocBlock_ = false;
//　时间戳交叉校验
double BaseMeasure::lastImuStamp_ = 0.0;
double BaseMeasure::lastVehStamp_ = 0.0;
double BaseMeasure::lastRtkStamp_ = 0.0;
double BaseMeasure::lastVlocStamp_ = 0.0;
// 物理量交叉校验
Eigen::Vector3d BaseMeasure::vLocPos_ = Eigen::Vector3d::Zero();
Eigen::Vector3d BaseMeasure::vLocAtt_ = Eigen::Vector3d::Zero();
Eigen::Vector3d BaseMeasure::rtkPos_ = Eigen::Vector3d::Zero();
Eigen::Vector3d BaseMeasure::rtkAtt_ = Eigen::Vector3d::Zero();
Eigen::Quaterniond BaseMeasure::stateQuat_ = Eigen::Quaterniond::Identity();
double BaseMeasure::vehVelo_ = 0.0;
// DataCheck
double BaseMeasure::vLocCheckOffset_ = 0.0;
bool BaseMeasure::useVlocCheckOffset_ = true;

void BaseMeasure::UpdateState(const State &state, State &nextState, const Eigen::VectorXd &dx, const Eigen::MatrixXd &mH,
      const Eigen::MatrixXd &mK) const
{
  nextState.q = state.q * Eigen::Quaterniond(1.0, dx(StateIndex::R) / 2.0, // 2.0
                                             dx(StateIndex::R + 1) / 2.0, dx(StateIndex::R + 2) / 2.0); // 2.0
  nextState.q.normalize();
  nextState.p += dx.segment<3>(StateIndex::P);
  nextState.v += dx.segment<3>(StateIndex::V);
  nextState.acc += dx.segment<3>(StateIndex::ACC);
  nextState.av += dx.segment<3>(StateIndex::AV);
  nextState.ba += dx.segment<3>(StateIndex::BA);
  nextState.bg += dx.segment<3>(StateIndex::BG);
  nextState.mP = (Eigen::MatrixXd::Identity(StateIndex::STATE_TOTAL, StateIndex::STATE_TOTAL) - mK * mH)
      * state.mP;
}

} // namespace Ego
} // namespace Localization
